/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.PowerManager;
import android.provider.Settings;
import android.view.Window;
import android.view.WindowManager;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.tool.DataUtils;

import java.io.IOException;

public class AlarmAlertActivity extends Activity implements OnClickListener, OnDismissListener {
    private long mNoteId;           // 存储笔记的 ID
    private String mSnippet;        // 存储笔记的简短内容（摘要）
    private static final int SNIPPET_PREW_MAX_LEN = 60; // 摘要最大长度
    MediaPlayer mPlayer;           // 用于播放闹铃音的媒体播放器

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE); // 请求无标题窗口

        final Window win = getWindow();
        win.addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED); // 当屏幕锁定时仍然显示窗口

        // 如果屏幕关闭，保持屏幕亮起
        if (!isScreenOn()) {
            win.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_LAYOUT_INSET_DECOR);
        }

        Intent intent = getIntent();  // 获取传递进来的 Intent

        try {
            mNoteId = Long.valueOf(intent.getData().getPathSegments().get(1)); // 获取笔记的 ID
            // 获取笔记的简短摘要，如果摘要太长，截取并附加说明
            mSnippet = DataUtils.getSnippetById(this.getContentResolver(), mNoteId);
            mSnippet = mSnippet.length() > SNIPPET_PREW_MAX_LEN ? mSnippet.substring(0,
                    SNIPPET_PREW_MAX_LEN) + getResources().getString(R.string.notelist_string_info)
                    : mSnippet;
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
            return; // 如果获取笔记信息失败，则返回
        }

        mPlayer = new MediaPlayer();  // 初始化媒体播放器
        // 如果笔记在数据库中是可见的，显示对话框并播放闹铃音
        if (DataUtils.visibleInNoteDatabase(getContentResolver(), mNoteId, Notes.TYPE_NOTE)) {
            showActionDialog();
            playAlarmSound();
        } else {
            finish();  // 如果笔记不可见，则结束此活动
        }
    }

    // 检查屏幕是否开启
    private boolean isScreenOn() {
        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
        return pm.isScreenOn();
    }

    // 播放闹铃音
    private void playAlarmSound() {
        // 获取系统的默认闹铃音 URI
        Uri url = RingtoneManager.getActualDefaultRingtoneUri(this, RingtoneManager.TYPE_ALARM);

        // 获取当前铃声模式，检查闹铃是否会被静音
        int silentModeStreams = Settings.System.getInt(getContentResolver(),
                Settings.System.MODE_RINGER_STREAMS_AFFECTED, 0);

        // 根据静音模式设置音频流类型
        if ((silentModeStreams & (1 << AudioManager.STREAM_ALARM)) != 0) {
            mPlayer.setAudioStreamType(silentModeStreams);
        } else {
            mPlayer.setAudioStreamType(AudioManager.STREAM_ALARM); // 设置音频流类型为闹铃
        }
        try {
            // 设置闹铃音频数据源，并准备播放
            mPlayer.setDataSource(this, url);
            mPlayer.prepare();
            mPlayer.setLooping(true); // 设置闹铃音循环播放
            mPlayer.start();  // 开始播放
        } catch (IllegalArgumentException | SecurityException | IllegalStateException | IOException e) {
            // 捕获并打印异常
            e.printStackTrace();
        }
    }

    // 显示提示操作对话框
    private void showActionDialog() {
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        dialog.setTitle(R.string.app_name);  // 设置对话框标题为应用名称
        dialog.setMessage(mSnippet);         // 显示笔记的简短摘要
        dialog.setPositiveButton(R.string.notealert_ok, this); // 设置“确定”按钮
        // 如果屏幕开启，设置“进入”按钮
        if (isScreenOn()) {
            dialog.setNegativeButton(R.string.notealert_enter, this);
        }
        dialog.show().setOnDismissListener(this); // 显示对话框并设置监听器
    }

    // 处理按钮点击事件
    public void onClick(DialogInterface dialog, int which) {
        switch (which) {
            case DialogInterface.BUTTON_NEGATIVE:
                // 如果点击了“进入”按钮，启动编辑笔记活动
                Intent intent = new Intent(this, NoteEditActivity.class);
                intent.setAction(Intent.ACTION_VIEW); // 设置意图为查看笔记
                intent.putExtra(Intent.EXTRA_UID, mNoteId); // 传递笔记 ID
                startActivity(intent); // 启动活动
                break;
            default:
                break;
        }
    }

    // 对话框关闭时停止播放闹铃音
    public void onDismiss(DialogInterface dialog) {
        stopAlarmSound();
        finish(); // 结束当前活动
    }

    // 停止播放闹铃音
    private void stopAlarmSound() {
        if (mPlayer != null) {
            mPlayer.stop();  // 停止播放
            mPlayer.release();  // 释放播放器资源
            mPlayer = null; // 清空播放器对象
        }
    }
}
