/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;

import android.content.Context;
import android.database.Cursor;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.Data;
import android.telephony.PhoneNumberUtils;
import android.util.Log;

import java.util.HashMap;

public class Contact {
    // 静态缓存，用于存储已查询过的电话号码对应的联系人姓名
    private static HashMap<String, String> sContactCache;
    // 用于日志记录的标签
    private static final String TAG = "Contact";

    // 查询条件，用于根据电话号码查询联系人姓名
    private static final String CALLER_ID_SELECTION = "PHONE_NUMBERS_EQUAL(" + Phone.NUMBER
    + ",?) AND " + Data.MIMETYPE + "='" + Phone.CONTENT_ITEM_TYPE + "'"
    + " AND " + Data.RAW_CONTACT_ID + " IN "
            + "(SELECT raw_contact_id "
            + " FROM phone_lookup"
            + " WHERE min_match = '+')";

    /**
     * 获取电话号码对应的联系人姓名
     * @param context 应用程序的上下文
     * @param phoneNumber 要查询的电话号码
     * @return 返回电话号码对应的联系人姓名，如果找不到则返回null
     */
    public static String getContact(Context context, String phoneNumber) {
        // 初始化缓存
        if(sContactCache == null) {
            sContactCache = new HashMap<String, String>();
        }

        // 如果缓存中已经有该电话号码的联系人信息，直接返回
        if(sContactCache.containsKey(phoneNumber)) {
            return sContactCache.get(phoneNumber);
        }

        // 使用电话号码生成查询条件
        String selection = CALLER_ID_SELECTION.replace("+",
                PhoneNumberUtils.toCallerIDMinMatch(phoneNumber));

        // 查询联系人信息
        Cursor cursor = context.getContentResolver().query(
                Data.CONTENT_URI,
                new String [] { Phone.DISPLAY_NAME },  // 需要获取的字段，这里是联系人姓名
                selection,  // 查询条件
                new String[] { phoneNumber },  // 查询参数，填充到查询条件中
                null);  // 排序方式，null表示不排序

        // 检查查询结果并返回联系人姓名
        if (cursor != null && cursor.moveToFirst()) {
            try {
                String name = cursor.getString(0);  // 获取查询到的联系人姓名
                sContactCache.put(phoneNumber, name);  // 将结果缓存起来
                return name;  // 返回联系人姓名
            } catch (IndexOutOfBoundsException e) {
                // 捕捉异常并记录错误日志
                Log.e(TAG, " Cursor get string error " + e.toString());
                return null;
            } finally {
                // 关闭游标，释放资源
                cursor.close();
            }
        } else {
            // 如果没有匹配的联系人，记录日志并返回null
            Log.d(TAG, "No contact matched with number:" + phoneNumber);
            return null;
        }
    }
}
