/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import java.util.Calendar;
import net.micode.notes.R;
import net.micode.notes.ui.DateTimePicker;
import net.micode.notes.ui.DateTimePicker.OnDateTimeChangedListener;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.text.format.DateFormat;
import android.text.format.DateUtils;

/**
 * 自定义的日期时间选择对话框。
 * 用于选择日期和时间，支持24小时制和12小时制显示。
 */
public class DateTimePickerDialog extends AlertDialog implements OnClickListener {

    // 当前选择的日期和时间
    private Calendar mDate = Calendar.getInstance();
    // 是否为24小时制
    private boolean mIs24HourView;
    // 日期时间选择回调
    private OnDateTimeSetListener mOnDateTimeSetListener;
    // 日期时间选择器控件
    private DateTimePicker mDateTimePicker;

    /**
     * 日期时间设置回调接口。
     * 当用户选择完日期时间后，回调此方法。
     */
    public interface OnDateTimeSetListener {
        void OnDateTimeSet(AlertDialog dialog, long date);
    }

    /**
     * 构造函数，初始化日期时间选择对话框。
     * @param context 上下文
     * @param date 初始日期时间
     */
    public DateTimePickerDialog(Context context, long date) {
        super(context);

        // 创建 DateTimePicker 控件，并设置监听器
        mDateTimePicker = new DateTimePicker(context);
        setView(mDateTimePicker);
        mDateTimePicker.setOnDateTimeChangedListener(new OnDateTimeChangedListener() {
            // 日期时间改变监听器
            public void onDateTimeChanged(DateTimePicker view, int year, int month,
                    int dayOfMonth, int hourOfDay, int minute) {
                // 更新选中的日期和时间
                mDate.set(Calendar.YEAR, year);
                mDate.set(Calendar.MONTH, month);
                mDate.set(Calendar.DAY_OF_MONTH, dayOfMonth);
                mDate.set(Calendar.HOUR_OF_DAY, hourOfDay);
                mDate.set(Calendar.MINUTE, minute);
                // 更新对话框标题显示的日期时间
                updateTitle(mDate.getTimeInMillis());
            }
        });

        // 设置初始日期时间
        mDate.setTimeInMillis(date);
        mDate.set(Calendar.SECOND, 0); // 将秒数设置为0
        mDateTimePicker.setCurrentDate(mDate.getTimeInMillis()); // 初始化日期时间选择器

        // 设置对话框按钮
        setButton(context.getString(R.string.datetime_dialog_ok), this); // 确定按钮
        setButton2(context.getString(R.string.datetime_dialog_cancel), (OnClickListener)null); // 取消按钮

        // 设置是否为24小时制
        set24HourView(DateFormat.is24HourFormat(this.getContext()));

        // 更新对话框标题
        updateTitle(mDate.getTimeInMillis());
    }

    /**
     * 设置是否为24小时制
     * @param is24HourView 是否为24小时制
     */
    public void set24HourView(boolean is24HourView) {
        mIs24HourView = is24HourView;
    }

    /**
     * 设置日期时间设置回调监听器
     * @param callBack 回调接口
     */
    public void setOnDateTimeSetListener(OnDateTimeSetListener callBack) {
        mOnDateTimeSetListener = callBack;
    }

    /**
     * 更新对话框标题，显示当前选择的日期和时间
     * @param date 当前选择的日期时间
     */
    private void updateTitle(long date) {
        // 设置显示的日期格式，包含年、月、日和时间
        int flag =
            DateUtils.FORMAT_SHOW_YEAR |
            DateUtils.FORMAT_SHOW_DATE |
            DateUtils.FORMAT_SHOW_TIME;
        // 如果是24小时制，更新显示格式
        flag |= mIs24HourView ? DateUtils.FORMAT_24HOUR : DateUtils.FORMAT_12HOUR;
        // 设置对话框标题为格式化后的日期时间
        setTitle(DateUtils.formatDateTime(this.getContext(), date, flag));
    }

    /**
     * 确定按钮的点击事件处理
     * 当用户点击确定按钮时，调用回调接口
     */
    public void onClick(DialogInterface arg0, int arg1) {
        if (mOnDateTimeSetListener != null) {
            // 回调监听器，传递选中的日期时间
            mOnDateTimeSetListener.OnDateTimeSet(this, mDate.getTimeInMillis());
        }
    }
}
