/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;

import android.app.SearchManager;
import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Intent;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

import net.micode.notes.R;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.NotesDatabaseHelper.TABLE;

/**
 * NotesProvider是内容提供者类，负责提供与笔记数据的增删改查功能。 
 * 它通过SQLite数据库进行数据存储，并使用UriMatcher来解析URI请求。
 */
public class NotesProvider extends ContentProvider {

    // 定义UriMatcher对象，用于匹配不同类型的URI
    private static final UriMatcher mMatcher;

    // Notes数据库帮助类，用于操作数据库
    private NotesDatabaseHelper mHelper;

    // 日志TAG
    private static final String TAG = "NotesProvider";

    // 定义不同的URI匹配码
    private static final int URI_NOTE = 1;           // 匹配所有笔记
    private static final int URI_NOTE_ITEM = 2;      // 匹配单个笔记
    private static final int URI_DATA = 3;           // 匹配所有数据
    private static final int URI_DATA_ITEM = 4;      // 匹配单个数据
    private static final int URI_SEARCH = 5;         // 匹配搜索请求
    private static final int URI_SEARCH_SUGGEST = 6; // 匹配搜索建议请求

    // 静态代码块，用于初始化UriMatcher
    static {
        mMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        mMatcher.addURI(Notes.AUTHORITY, "note", URI_NOTE); // 匹配笔记URI
        mMatcher.addURI(Notes.AUTHORITY, "note/#", URI_NOTE_ITEM); // 匹配具体的笔记
        mMatcher.addURI(Notes.AUTHORITY, "data", URI_DATA); // 匹配数据URI
        mMatcher.addURI(Notes.AUTHORITY, "data/#", URI_DATA_ITEM); // 匹配具体的数据
        mMatcher.addURI(Notes.AUTHORITY, "search", URI_SEARCH); // 匹配搜索
        mMatcher.addURI(Notes.AUTHORITY, SearchManager.SUGGEST_URI_PATH_QUERY, URI_SEARCH_SUGGEST); // 搜索建议
        mMatcher.addURI(Notes.AUTHORITY, SearchManager.SUGGEST_URI_PATH_QUERY + "/*", URI_SEARCH_SUGGEST);
    }

    /**
     * SQL查询片段，用于从笔记中搜索，删除换行符并裁剪空格，以显示更多信息。
     */
    private static final String NOTES_SEARCH_PROJECTION = NoteColumns.ID + ","
        + NoteColumns.ID + " AS " + SearchManager.SUGGEST_COLUMN_INTENT_EXTRA_DATA + ","
        + "TRIM(REPLACE(" + NoteColumns.SNIPPET + ", x'0A','')) AS " + SearchManager.SUGGEST_COLUMN_TEXT_1 + ","
        + "TRIM(REPLACE(" + NoteColumns.SNIPPET + ", x'0A','')) AS " + SearchManager.SUGGEST_COLUMN_TEXT_2 + ","
        + R.drawable.search_result + " AS " + SearchManager.SUGGEST_COLUMN_ICON_1 + ","
        + "'" + Intent.ACTION_VIEW + "' AS " + SearchManager.SUGGEST_COLUMN_INTENT_ACTION + ","
        + "'" + Notes.TextNote.CONTENT_TYPE + "' AS " + SearchManager.SUGGEST_COLUMN_INTENT_DATA;

    // SQL查询，用于搜索笔记内容中的摘要
    private static String NOTES_SNIPPET_SEARCH_QUERY = "SELECT " + NOTES_SEARCH_PROJECTION
        + " FROM " + TABLE.NOTE
        + " WHERE " + NoteColumns.SNIPPET + " LIKE ?"
        + " AND " + NoteColumns.PARENT_ID + "<>" + Notes.ID_TRASH_FOLER
        + " AND " + NoteColumns.TYPE + "=" + Notes.TYPE_NOTE;

    /**
     * 创建ContentProvider实例时调用。
     * 主要用于初始化数据库帮助类实例。
     */
    @Override
    public boolean onCreate() {
        mHelper = NotesDatabaseHelper.getInstance(getContext());
        return true;
    }

    /**
     * 处理查询请求。
     * 
     * @param uri           查询的URI
     * @param projection    查询的列
     * @param selection     查询的条件
     * @param selectionArgs 查询条件的参数
     * @param sortOrder     排序方式
     * @return 返回Cursor，表示查询结果
     */
    @Override
    public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {
        Cursor c = null;
        SQLiteDatabase db = mHelper.getReadableDatabase();
        String id = null;
        
        // 根据URI匹配码执行不同的查询操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE: // 查询所有笔记
                c = db.query(TABLE.NOTE, projection, selection, selectionArgs, null, null, sortOrder);
                break;
            case URI_NOTE_ITEM: // 查询单个笔记
                id = uri.getPathSegments().get(1);
                c = db.query(TABLE.NOTE, projection, NoteColumns.ID + "=" + id + parseSelection(selection), selectionArgs, null, null, sortOrder);
                break;
            case URI_DATA: // 查询所有数据
                c = db.query(TABLE.DATA, projection, selection, selectionArgs, null, null, sortOrder);
                break;
            case URI_DATA_ITEM: // 查询单个数据
                id = uri.getPathSegments().get(1);
                c = db.query(TABLE.DATA, projection, DataColumns.ID + "=" + id + parseSelection(selection), selectionArgs, null, null, sortOrder);
                break;
            case URI_SEARCH: // 搜索笔记
            case URI_SEARCH_SUGGEST: // 搜索建议
                if (sortOrder != null || projection != null) {
                    throw new IllegalArgumentException("do not specify sortOrder, selection, selectionArgs, or projection" + "with this query");
                }
                String searchString = null;
                if (mMatcher.match(uri) == URI_SEARCH_SUGGEST) {
                    if (uri.getPathSegments().size() > 1) {
                        searchString = uri.getPathSegments().get(1);
                    }
                } else {
                    searchString = uri.getQueryParameter("pattern");
                }
                if (TextUtils.isEmpty(searchString)) {
                    return null;
                }
                try {
                    searchString = String.format("%%%s%%", searchString); // 添加通配符进行模糊查询
                    c = db.rawQuery(NOTES_SNIPPET_SEARCH_QUERY, new String[] { searchString });
                } catch (IllegalStateException ex) {
                    Log.e(TAG, "got exception: " + ex.toString());
                }
                break;
            default:
                throw new IllegalArgumentException("Unknown URI " + uri); // 未知的URI类型
        }
        
        // 如果Cursor不为空，设置通知URI，监听数据变化
        if (c != null) {
            c.setNotificationUri(getContext().getContentResolver(), uri);
        }
        return c;
    }

    /**
     * 处理插入请求。
     * 
     * @param uri    插入的URI
     * @param values 要插入的数据
     * @return 返回新插入记录的URI
     */
    @Override
    public Uri insert(Uri uri, ContentValues values) {
        SQLiteDatabase db = mHelper.getWritableDatabase();
        long dataId = 0, noteId = 0, insertedId = 0;

        // 根据URI匹配码执行不同的插入操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE: // 插入一条笔记
                insertedId = noteId = db.insert(TABLE.NOTE, null, values);
                break;
            case URI_DATA: // 插入一条数据
                if (values.containsKey(DataColumns.NOTE_ID)) {
                    noteId = values.getAsLong(DataColumns.NOTE_ID);
                } else {
                    Log.d(TAG, "Wrong data format without note id:" + values.toString());
                }
                insertedId = dataId = db.insert(TABLE.DATA, null, values);
                break;
            default:
                throw new IllegalArgumentException("Unknown URI " + uri); // 未知的URI类型
        }

        // 通知数据改变
        if (noteId > 0) {
            getContext().getContentResolver().notifyChange(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId), null);
        }
        if (dataId > 0) {
            getContext().getContentResolver().notifyChange(ContentUris.withAppendedId(Notes.CONTENT_DATA_URI, dataId), null);
        }
        return ContentUris.withAppendedId(uri, insertedId); // 返回新记录的URI
    }

    /**
     * 处理删除请求。
     * 
     * @param uri           删除的URI
     * @param selection     删除的条件
     * @param selectionArgs 删除条件的参数
     * @return 返回删除的记录数
     */
    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        int count = 0;
        String id = null;
        SQLiteDatabase db = mHelper.getWritableDatabase();
        boolean deleteData = false;

        // 根据URI匹配码执行不同的删除操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE: // 删除笔记
                selection = "(" + selection + ") AND " + NoteColumns.ID + ">0 ";
                count = db.delete(TABLE.NOTE, selection, selectionArgs);
                break;
            case URI_NOTE_ITEM: // 删除单个笔记
                id = uri.getPathSegments().get(1);
                long noteId = Long.valueOf(id);
                if (noteId <= 0) { // 如果笔记ID小于等于0，不允许删除
                    break;
                }
                count = db.delete(TABLE.NOTE, NoteColumns.ID + "=" + id + parseSelection(selection), selectionArgs);
                break;
            case URI_DATA: // 删除数据
                count = db.delete(TABLE.DATA, selection, selectionArgs);
                deleteData = true;
                break;
            case URI_DATA_ITEM: // 删除单个数据
                id = uri.getPathSegments().get(1);
                count = db.delete(TABLE.DATA, DataColumns.ID + "=" + id + parseSelection(selection), selectionArgs);
                deleteData = true;
                break;
            default:
                throw new IllegalArgumentException("Unknown URI " + uri); // 未知的URI类型
        }

        // 如果删除成功，通知数据变化
        if (count > 0) {
            if (deleteData) {
                getContext().getContentResolver().notifyChange(Notes.CONTENT_NOTE_URI, null);
            }
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return count;
    }

    /**
     * 处理更新请求。
     * 
     * @param uri           更新的URI
     * @param values        更新的内容
     * @param selection     更新的条件
     * @param selectionArgs 更新条件的参数
     * @return 返回更新的记录数
     */
    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        int count = 0;
        String id = null;
        SQLiteDatabase db = mHelper.getWritableDatabase();
        boolean updateData = false;

        // 根据URI匹配码执行不同的更新操作
        switch (mMatcher.match(uri)) {
            case URI_NOTE: // 更新笔记
                increaseNoteVersion(-1, selection, selectionArgs); // 更新笔记版本
                count = db.update(TABLE.NOTE, values, selection, selectionArgs);
                break;
            case URI_NOTE_ITEM: // 更新单个笔记
                id = uri.getPathSegments().get(1);
                increaseNoteVersion(Long.valueOf(id), selection, selectionArgs); // 更新指定笔记版本
                count = db.update(TABLE.NOTE, values, NoteColumns.ID + "=" + id + parseSelection(selection), selectionArgs);
                break;
            case URI_DATA: // 更新数据
                count = db.update(TABLE.DATA, values, selection, selectionArgs);
                updateData = true;
                break;
            case URI_DATA_ITEM: // 更新单个数据
                id = uri.getPathSegments().get(1);
                count = db.update(TABLE.DATA, values, DataColumns.ID + "=" + id + parseSelection(selection), selectionArgs);
                updateData = true;
                break;
            default:
                throw new IllegalArgumentException("Unknown URI " + uri); // 未知的URI类型
        }

        // 如果更新成功，通知数据变化
        if (count > 0) {
            if (updateData) {
                getContext().getContentResolver().notifyChange(Notes.CONTENT_NOTE_URI, null);
            }
            getContext().getContentResolver().notifyChange(uri, null);
        }
        return count;
    }

    /**
     * 辅助方法，解析选择条件。
     * 
     * @param selection 选择条件
     * @return 返回解析后的选择条件
     */
    private String parseSelection(String selection) {
        return (!TextUtils.isEmpty(selection) ? " AND (" + selection + ')' : "");
    }

    /**
     * 更新笔记的版本号。
     * 
     * @param id            笔记ID
     * @param selection     选择条件
     * @param selectionArgs 选择条件参数
     */
    private void increaseNoteVersion(long id, String selection, String[] selectionArgs) {
        StringBuilder sql = new StringBuilder(120);
        sql.append("UPDATE ");
        sql.append(TABLE.NOTE);
        sql.append(" SET ");
        sql.append(NoteColumns.VERSION);
        sql.append("=" + NoteColumns.VERSION + "+1 ");

        if (id > 0 || !TextUtils.isEmpty(selection)) {
            sql.append(" WHERE ");
        }
        if (id > 0) {
            sql.append(NoteColumns.ID + "=" + String.valueOf(id));
        }
        if (!TextUtils.isEmpty(selection)) {
            String selectString = id > 0 ? parseSelection(selection) : selection;
            for (String args : selectionArgs) {
                selectString = selectString.replaceFirst("\\?", args);
            }
            sql.append(selectString);
        }

        // 执行SQL更新版本
        mHelper.getWritableDatabase().execSQL(sql.toString());
    }

    @Override
    public String getType(Uri uri) {
        // TODO Auto-generated method stub
        return null;
    }
}
